#!/user/freepool/local/bin/python2.2
# -*- coding: latin1 -*-
"""\
Das Programm %(progname)s korrigiert den mibrauchten Kontrollflu von Testbenches
der auf Assertions mit 'error' und 'failure' beruht. Diese werden durch 'note' ersetzt.
Voraussetzung ist, da das Muster den der unten verwiesenen Links entspricht.

Aufruf:
    %(progname)s [<architecture-path> ... ] [-h] [--help]

Ist kein Archtekturpfad angegeben, wird das Ergebnis von stdin gelesen und
nach stdout geschrieben. Andernfalls wird die Architekturdatei gepatcht. Eine
Backup-Kopie wird nach <architecture-path>.bak geschrieben.

siehe auch: 
    file:/user/rxpro5/rxpro5/util/import_lists/__REPARIERE_ASSERT_FAILURE__.txt
    
Beispieldatei:
    file:/user/freepool/libgrafen/python/grafen/tb_ar_fix_controlflow/mult10x6_tb1-a0-ar.vhd
"""

"""
    IF (not endfile(stim)) THEN
      readline(stim,zeile_stim);
    ELSE
      assert false report "****** STIMULIFILE LEER GELESEN ******" severity note; -- 'note' statt 'failure'
    END if;  

    IF (not endfile(resp)) THEN
      readline(resp,zeile_resp);
    ELSE
      run_simulation <= FALSE;     -- hier enfgen
      exit ;
    END if;  
   end loop;
   assert false report "****** SIMULATION BEENDET ******" severity note ; -- 'note' statt 'failure'
"""
 
#import re
import pre as re
import sys,os
import stat
import indent

re_insert_signal = re.compile(
            "^(.*?\s*\n\s*signal.*?)"
                "(?P<insert_signal>\n)"
            "(\s*begin"
    ".*?\s"
    "inclke\s*\<=\s*not\s+inclke\s+after\s+0\.5\s*\*\s*PERIODE"
    ")"
    "(?P<ext_inclke>\s*;)"
    "("
    ".*?\s"
    "IF\s+\(not\s+endfile\s*\(\s*stim\s*\)\s*\)\s*THEN\s+"
    "readline\s*\(\s*stim\s*,\s*zeile_stim\s*\)\s*;\s*"
    "ELSE\s+"
      'assert\s+false\s+report\s+"\*+\s+STIMULIFILE\s+LEER\s+GELESEN\s+\*+"\s+'
      'severity\s+'
    ")"
    "(?P<exception1>(?:failure|error))"
    "("
    "\s+;\s+"
    "END\s+if\s*;\s*"  
    ".*?"
    "IF\s*\(not\s+endfile\s*\(\s*resp\s*\)\s*\)\s*THEN\s+"
    "readline\s*\(\s*resp\s*,\s*zeile_resp\)\s*;\s*"
    "ELSE\s*?"
    ")"
    "(?P<insert_run_simulation>\n)"
    "("
    "\s*"
    "exit\s*;\s*"
    "END\s+if\s*;\s*"
    "end\s+loop\s*;\s*"
    'assert\s+false\s+report\s+"\*+\s+SIMULATION\s+BEENDET\s+\*+"\s+'
    'severity\s+'
    ")"
    "(?P<exception2>(?:error|failure))\s*"
    "("
    ";\s*" 
             ".*)\Z",re.DOTALL |re.I)
"""
# \1 other 1
# \2 insert_signal
# \3 other 2
# \4 extend_inclke
# \5 other 3
# \6 exception1
# \7 other 4
# \8 insert_run_simulation
# \9 other 5
# \10 exception2
# \11 other 6
#    "  run_simulation <= FALSE;"    -- hier enfgen
"""

def update(text):
    replaced = re.sub(
        re_insert_signal,
        (
            r'\1\n'
            r'signal   run_simulation : boolean := TRUE;\n'
            r'\3'
            r' when run_simulation = TRUE else\n          inclke;'
            r'\5'
            r'note'
            r'\7\n'
            r'      run_simulation <= FALSE;\n'
            r'\9'
            r'note'
            r'\11'
        )
        ,text)

    return replaced

if __name__ == '__main__':
    progname = os.path.basename(sys.argv[0])

    arglen = len(sys.argv)
    if arglen == 1:
        text = sys.stdin.read()

        replaced = update(text)

        if replaced == text:
            print >> sys.stderr,"### Warnung: Text nicht modifiziert!"

        print replaced
        sys.exit(0)


    elif '-h' in sys.argv[1:] or '--help' in sys.argv[1:]:
        txt = __doc__ % vars()
        template = indent.template("""\
Hilfe fr %(progname)s:
    %(txt)s
""")
        print template % vars()

    else:
        error = 0
        for fname in sys.argv[1:]:
            fp = open(fname)

            text = fp.read()

            replaced = update(text)

            if fp == sys.stdin:
                print replaced

            else:
                fp.close()
                if replaced == text:
                    print "Info: Text nicht modifiziert!"

                else:
                    if not os.stat(fname).st_mode & stat.S_IWUSR:
                        print "### Fehler: Schreiben nicht erlaubt auf %s" % fname
                        error += 1
                        continue

                    fname_bak = fname+'.bak'

                    if os.path.exists(fname_bak):
                        os.remove(fname_bak)

                    os.rename(fname,fname+'.bak')

                    open(fname,'w').write(replaced)

                    print "Info: '%s' Kontrollflu korrigiert, Backup Kopie in '%s'" % (
                            fname,fname_bak)

        sys.exit(error)
